@@ -3,7 +3,7 @@
 #
 # Copyright (C) 1998, 1999 Albert Hopkins (marduk)
 # Copyright (C) 2002 Mike Meyer
-# Copyright (C) 2005, 2006 Arthur de Jong
+# Copyright (C) 2005, 2006, 2007 Arthur de Jong
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -30,7 +30,7 @@ items should be changeble from the command line."""
 import urllib
 
 # Current version of webcheck.
-VERSION = '1.9.7'
+VERSION = '1.10.0'
 
 # The homepage of webcheck.
 HOMEPAGE = 'http://ch.tudelft.nl/~arthur/webcheck/'
@@ -110,3 +110,7 @@ FILE_INDEXES = [ 'index.html', 'index.htm' ]
 # A list of names that will be checked when encountering an ftp://
 # directory. This file will be picked up instead of the directory list.
 FTP_INDEXES = [ 'index.html', 'index.htm' ]
+
+# Whether to fetch robots.txt files and do checking based on the information
+# present in those files (normally matching links are yanked).
+USE_ROBOTS = True
