@@ -3,7 +3,7 @@
 #
 # Copyright (C) 1998, 1999 Albert Hopkins (marduk)
 # Copyright (C) 2002 Mike W. Meyer
-# Copyright (C) 2005, 2006 Arthur de Jong
+# Copyright (C) 2005, 2006, 2007 Arthur de Jong
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -95,7 +95,7 @@ class Site:
     The available properties of this class are:
 
       linkMap    - a map of urls to link objects
-      base       - a url that points to the base of the site
+      bases      - a list of base link object
    """
 
     def __init__(self):
@@ -113,8 +113,8 @@ class Site:
         self._robotparsers = {}
         # a map of urls to Link objects
         self.linkMap = {}
-        # base url that can be used as start of site
-        self.base = None
+        # list of base urls (these are the internal urls to start from)
+        self.bases = []
 
     def add_internal(self, url):
         """Add the given url and consider all urls below it to be internal.
@@ -204,6 +204,9 @@ class Site:
         # check if we should avoid external links
         if not link.isinternal and config.AVOID_EXTERNAL_LINKS:
             return 'external avoided'
+        # check if we should use robot parsers
+        if not config.USE_ROBOTS:
+            return False
         # skip schemes not haveing robot.txt files
         if link.scheme != 'http' and link.scheme != 'https':
             return False
@@ -291,25 +294,22 @@ class Site:
     def postprocess(self):
         # build the list of urls that were set up with add_internal() that
         # do not have a parent (they form the base for the site)
-        bases = []
         for url in self._internal_urls:
             link = self.linkMap[url].follow_link()
             if link == None:
                 debugio.warn('base link %s redirects to nowhere' % url)
                 continue
-            # if the link has no parent add it to the result list,
-            # unless it is the first one
-            if len(link.parents) == 0 or len(bases) == 0:
-                debugio.debug('crawler.postprocess(): adding %s to bases' % link.url)
-                bases.append(link)
+            # add the link to bases
+            debugio.debug('crawler.postprocess(): adding %s to bases' % link.url)
+            self.bases.append(link)
         # if we got no bases, just use the first internal one
-        if len(bases) == 0:
+        if len(self.bases) == 0:
             debugio.debug('crawler.postprocess(): fallback to adding %s to bases' % self._internal_urls[0])
-            bases.append(self.linkMap[self._internal_urls[0]])
+            self.bases.append(self.linkMap[self._internal_urls[0]])
         # do a breadth first traversal of the website to determin depth and
         # figure out page children
         tocheck = []
-        for link in bases:
+        for link in self.bases:
             link.depth = 0
             tocheck.append(link)
         # repeat until we have nothing more to check
@@ -325,7 +325,7 @@ class Site:
                 tocheck.append(child)
         # set some compatibility properties
         # TODO: figure out a better way to get to this to the plugins
-        self.base = bases[0].url
+        self.base = self.bases[0].url
 
 class Link:
     """This is a basic class representing a url.
@@ -410,9 +410,10 @@ class Link:
         self.redirectlist = None
         self._ischanged = False
 
-    def _checkurl(self, url):
+    def __checkurl(self, url):
         """Check to see if the url is formatted properly, correct formatting
         if possible and log an error in the formatting to the current page."""
+        # search for spaces in the url
         if _spacepattern.search(url):
             self.add_pageproblem('link contains unescaped spaces: %s' % url)
             # replace spaces by %20
@@ -430,15 +431,32 @@ class Link:
             pass
         return url
 
+    def __tolink(self, link):
+        """Convert the link to a link object, either it is already a link,
+        a link object is returned from the database or a new link is
+        created. This returns None for empty strings."""
+        # ignore if child is empty string
+        if link == '' or link == u'':
+            return None
+        if type(link) is unicode and self.encoding:
+            # convert url to binary if passed as unicode
+            link = link.encode(self.encoding)
+        # convert the url to a link object if we were called with a url
+        if type(link) is unicode or type(link) is str:
+            link = self.site.get_link(self.__checkurl(link))
+        # re're done
+        return link
+
     def add_child(self, child):
         """Add a link object to the child relation of this link.
         The reverse relation is also made."""
         # ignore children for external links
         if not self.isinternal:
             return
-        # convert the url to a link object if we were called with a url
-        if type(child) is str:
-            child = self.site.get_link(self._checkurl(child))
+        # convert to link object
+        child = self.__tolink(child)
+        if child is None:
+            return
         # add to children
         if child not in self.children:
             self.children.append(child)
@@ -452,9 +470,10 @@ class Link:
         # ignore embeds for external links
         if not self.isinternal:
             return
-        # convert the url to a link object if we were called with a url
-        if type(link) is str:
-            link = self.site.get_link(self._checkurl(link))
+        # convert to link object
+        link = self.__tolink(link)
+        if link is None:
+            return
         # add to embedded
         if link not in self.embedded:
             self.embedded.append(link)
@@ -465,10 +484,12 @@ class Link:
 
     def add_anchor(self, anchor):
         """Indicate that this page contains the specified anchor."""
+        # lowercase anchor
+        anchor = anchor.lower()
         # add anchor
         if anchor in self.anchors:
             self.add_pageproblem(
-              'anchor "%(anchor)s" defined multiple times'
+              'anchor/id "%(anchor)s" defined multiple times'
               % { 'anchor':   anchor })
         else:
             self.anchors.append(anchor)
@@ -477,9 +498,11 @@ class Link:
     def add_reqanchor(self, parent, anchor):
         """Indicate that the specified link contains a reference to the
         specified anchor. This can be checked later."""
+        # lowercase anchor
+        anchor = anchor.lower()
         # convert the url to a link object if we were called with a url
         if type(parent) is str:
-            parent = self.site.get_link(self._checkurl(parent))
+            parent = self.site.get_link(self.__checkurl(parent))
         # add anchor
         if anchor in self.reqanchors:
             if parent not in self.reqanchors[anchor]:
@@ -508,7 +531,7 @@ class Link:
             self.add_linkproblem('too many redirects (%d)' % self.redirectdepth)
             return None
         # check for redirect to self
-        url = self._checkurl(url)
+        url = self.__checkurl(url)
         if url == self.url:
             self.add_linkproblem('redirect same as source: %s' % url)
             return None
@@ -566,10 +589,13 @@ class Link:
         """If this link represents a redirect return the redirect target,
         otherwise return self. If this redirect does not find a referenced
         link None is returned."""
+        # if this is not a redirect just return
         if self.redirectdepth == 0:
             return self
+        # if we don't know where this redirects, return None
         if len(self.children) == 0:
             return None
+        # len(self.children) should be 1!
         # set up visited
         if visited is None:
             visited = []
@@ -577,6 +603,7 @@ class Link:
         visited.append(self)
         if self.children[0] in visited:
             return None
+        # check where we redirect to
         return self.children[0].follow_link(visited)
 
     def _pagechildren(self):
@@ -613,3 +640,14 @@ class Link:
                 self.pagechildren.append(child)
         # return the results
         return self.pagechildren
+
+    def set_encoding(self, encoding):
+        """Set the encoding of the link doing some basic checks
+        to see if the encoding is supported."""
+        if self.encoding is None:
+            try:
+                debugio.debug('crawler.Link.set_encoding("'+str(encoding)+'")')
+                unicode('just some random text', encoding, 'replace')
+                self.encoding = encoding
+            except Exception:
+                self.add_pageproblem('unknown encoding: ' + encoding)
