@@ -3,7 +3,7 @@
 #
 # Copyright (C) 1998, 1999 Albert Hopkins (marduk)
 # Copyright (C) 2002 Mike W. Meyer
-# Copyright (C) 2005, 2006 Arthur de Jong
+# Copyright (C) 2005, 2006, 2007 Arthur de Jong
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -49,6 +49,11 @@ def fetch(link, acceptedtypes):
     (userpass, netloc) = urllib.splituser(link.netloc)
     proxyuserpass = None
     scheme = link.scheme
+    # check validity of netloc (to work around bug in idna module)
+    if netloc[0] == '.':
+        debugio.debug('schemes.http.fetch(): fail on hostname starting with dot')
+        link.add_linkproblem('hostname starts with a dot')
+        return None
     # check which host to connect to (if using proxies)
     if config.PROXIES and config.PROXIES.has_key(link.scheme):
         # pass the complete url in the request, connecting to the proxy
@@ -76,12 +81,12 @@ def fetch(link, acceptedtypes):
                 (user, passwd) = urllib.splitpasswd(userpass)
                 conn.putheader(
                   'Authorization',
-                  'Basic '+base64.encodestring(user+':'+passwd).strip() )
+                  'Basic '+base64.encodestring(str(user)+':'+str(passwd)).strip() )
             if proxyuserpass is not None:
                 (user, passwd) = urllib.splitpasswd(proxyuserpass)
                 conn.putheader(
                   'Proxy-Authorization',
-                  'Basic '+base64.encodestring(user+':'+passwd).strip() )
+                  'Basic '+base64.encodestring(str(user)+':'+str(passwd)).strip() )
             # bypass proxy cache
             if config.BYPASSHTTPCACHE:
                 conn.putheader('Cache-control', 'no-cache')
@@ -105,7 +110,7 @@ def fetch(link, acceptedtypes):
             except AttributeError:
                 pass
             try:
-                link.encoding = _charsetpattern.search(response.getheader('Content-type')).group(1)
+                link.set_encoding(_charsetpattern.search(response.getheader('Content-type')).group(1))
             except (AttributeError, TypeError):
                 pass
             try:
@@ -142,7 +147,7 @@ def fetch(link, acceptedtypes):
             debugio.debug('error reading HTTP response: '+str(e))
             link.add_linkproblem('error reading HTTP response: '+str(e))
             return None
-        except socket.error, e:
+        except (socket.error, socket.sslerror), e:
             if hasattr(e, 'args') and len(e.args) == 2:
                 debugio.debug("error reading HTTP response: "+str(e.args[1]))
                 link.add_linkproblem("error reading HTTP response: "+str(e.args[1]))
@@ -150,6 +155,13 @@ def fetch(link, acceptedtypes):
                 debugio.debug("error reading HTTP response: "+str(e))
                 link.add_linkproblem("error reading HTTP response: "+str(e))
             return None
+        except Exception, e:
+            # handle all other exceptions
+            debugio.debug('unknown exception caught: '+str(e))
+            link.add_linkproblem('error reading HTTP response: '+str(e))
+            import traceback
+            traceback.print_exc()
+            return None
     finally:
         # close the connection before returning
         if conn is not None:
